#include "Drafo.h"

bool Drafo::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e15: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string clearing(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
// Line-1
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(clearing);
        }

    } else {
        const string LINE_1_ERROR("e16(Line-1): in.getline(c, BUFFER_SIZE) ");
        error = LINE_1_ERROR;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string s;

    readingCsvToProduceArrays();
    if(whoText) {
        const string who_text(WHO_TEXT);
        saveName = who_text;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            listing();
            clear(s);
            continue;
        }

        s = GAP + s;
        s = checkOneByOne(s); // Replacing "V" with "7"
        s = replaceDraftWithFormal(s); // Replacing " 7 " with " 07 " for example
        s = s.substr(1);
        add(s);
    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool Drafo::openSaveClose(std::string & fileName) {
    using namespace std;

// Trimming
    string s = buffer->str();
    int size = s.length();
    string c;
    for(int i = 0; i < size; i += NEW_LINE_LENGTH) {
        c = s.substr(i, NEW_LINE_LENGTH);
        if(NEW_LINE == c) {
            continue;
        } else {
            s = s.substr(i);
            break;
        }

    }

// Writing
    ofstream saveFile( fileName.c_str() );
    if( saveFile.fail() ) {
        error = "e17: ofstream saveFile( " + fileName + CLOSE_BRACKET;
        return false;
    }

    saveFile << s;
    saveFile.close();
    return true;
}

bool Drafo::readingCsvToProduceArrays() {
    using namespace std;

// Reading DRAFO_CSV to produce an array
    ifstream in( DRAFO_CSV.c_str() );
    if( in.fail() ) {
        error = "e18: ifstream in( " + DRAFO_CSV + CLOSE_BRACKET;
        return false;
    }

    bool secondField = false;
    bool yen = false;
    char *from;
    char *p;
    char *b;
    char c[TO_BUFFER_SIZE];
    char to[TO_BUFFER_SIZE];
    const int COMMA = ',';
    const int YEN = 0x5C;
    const int YEN_SIZE = 1;
    int comma1 = 0;
    int comma2;
    int lineLength;
    string *d = new string("");
    string *f = new string("");

    while( in.getline(c, TO_BUFFER_SIZE) ) {
        lineLength = strlen(c);
        for(int i = 0; i < lineLength; ++i) {
            if(yen) {
                yen = false;
                continue;
            }

            if(secondField) {
                if(c[i] == YEN) { // If YEN precedes a comma in CSV, the second field shall include the comma.
                    yen = true;
                } else if(c[i] == COMMA) {
                    comma2 = i;

                    c[comma2] = '\0';
                    from = & c[ comma1 + 1 ];
                    b = strchr(from, YEN);
                    if(b == NULL) {
                        strcpy(to, from);
                    } else {
                        *b = '\0';
                        strcpy(to, from);
                        b += YEN_SIZE; // The second field shall exclude YEN.
                        strcat(to, b);
                    }

                    delete f;
                    f = new string(to);
                    if( f->length() > 0 ) {
                        formal.push_back(*f);
                    } else {
                        formal.push_back(BELL);
                    }

                    secondField = false;
                }

            } else { // first field
                if(c[i] == YEN) { // If YEN precedes a comma in CSV, the first field shall include the comma.
                    yen = true;
                } else if(c[i] == COMMA) {
                    comma1 = i;

                    c[comma1] = '\0';
                    p = & c[0];
                    b = strchr(p, YEN);
                    if(b == NULL) {
                        strcpy(to, p);
                    } else {
                        *b = '\0';
                        strcpy(to, p);
                        b += YEN_SIZE; // The first field shall exclude YEN.
                        strcat(to, b);
                    }

                    delete d;
                    d = new string(to);
                    if( d->length() > 0 ) {
                        draft.push_back(*d);
                    } else {
                        draft.push_back(BELL);
                    }

                    secondField = true;
                }

            }

        }

    }

    in.close();
    delete d; delete f; 
    return true;
}

std::string Drafo::checkOneByOne(std::string & line) {
    using namespace std;

    bool zenkaku = false;
    char c[BUFFER_SIZE];
    char ascii[] = {'1', '\0'};
    char kanji[] = {'1', '2', '\0'};
    int i, k;
    int lineLength = line.length();
    string *p = new string("");
    stringstream os;
    unsigned char uc;

    strcpy(c, line.c_str() );
    for(k = 0; k < lineLength; ++k) {
        uc = (unsigned char)c[k];
        i = (int)uc;

        if(zenkaku) {
            zenkaku = false;
            kanji[1] = c[k];
            delete p;
            p = new string(kanji);
            ascii[0] = z[*p]; // typedef std::map<std::string, int> ZENKAKU_HANKAKU; ZENKAKU_HANKAKU z;
            if(ascii[0] == 0) {
                os << kanji;
            } else {
                os << ascii;
            }

            continue;
        }

        if(0x81 <= i && i <= 0xFC) {
            if(KANA_KUTEN <= i && i <= 0xDF) { // 
                zenkaku = false;
                switch(i) {
                case 0xA4: // 
                    os << GAP;
                    break;
                case 0xA5: // 
                    os << GAP;
                    break;
                default:
                    ascii[0] = c[k];
                    os << ascii;
                    break;
                }

            } else {
                zenkaku = true;
                kanji[0] = c[k];
            }

        } else {
            zenkaku = false;
            ascii[0] = c[k];
            os << ascii;
        }

    }

    delete p;
    return os.str();
}

std::string Drafo::replaceDraftWithFormal(std::string & s) {
    using namespace std;

    int q, thatLength;
    string head, tail, that;
    string line = s;

    ITEM::iterator k = formal.begin(); 
    for(ITEM::iterator i = draft.begin(); i != draft.end(); ++i, ++k) {
        that = *i;
        thatLength = that.length();

        tail = line;
        line = EMPTY;
        q = tail.find(that);
        while(q != string::npos) {
            head = tail.substr(0, q);
            q += thatLength;
            tail = tail.substr(q);

            line += head;
            line += *k;
            q = tail.find(that);
        }

        line += tail;
    }

    return line;
}

Drafo::Drafo(std::string & d, bool f, bool s, bool w): ALL_QP("ALL QP"), BELL("\\a"), CLOSE_BRACKET(" )"), DRAFO_CSV("drafo.csv"), EMPTY(""), GAP(" "), NEW_LINE("\n"), NEW_LINE_LENGTH(1), NO_SPLITTING(true) {
    using namespace std;

    drafoCsv = d;
    firstLine = f;
    serialNumber = s;
    whoText = w;

    z["@"] = ' ';
    z["C"] = ' ';
    z["A"] = ' ';
    z["E"] = ' ';
    z["O"] = '0';
    z["P"] = '1';
    z["Q"] = '2';
    z["R"] = '3';
    z["S"] = '4';
    z["T"] = '5';
    z["U"] = '6';
    z["V"] = '7';
    z["W"] = '8';
    z["X"] = '9';
    z[""] = 'A';
    z[""] = 'L';
    z[""] = 'P';
    z[""] = 'Q';
    z["`"] = 'A';
    z["k"] = 'L';
    z["o"] = 'P';
    z["p"] = 'Q';
    z["b"] = '|';
    z["^"] = '/';

    ifstream test( DRAFO_CSV.c_str() );
    if( test.fail() ) {
        createDrafoCsv();
    } else {
        test.close();
    }

    buffer = new stringstream();
    combos = new stringstream();
    whose = EMPTY;
}
