#include "GizeMask.h"

extern GIZE_CODE *gc;

bool GizeMask::openSaveClose(std::string & fileName) {
    using namespace std;

// Trimming
    string s = buffer->str();
    int size = s.length();
    string c;
    for(int i = 0; i < size; i += NEW_LINE_LENGTH) {
        c = s.substr(i, NEW_LINE_LENGTH);
        if(NEW_LINE == c) {
            continue;
        } else {
            s = s.substr(i);
            break;
        }

    }

// Writing
    ofstream saveFile( fileName.c_str() );
    if( saveFile.fail() ) {
        error = "e3: ofstream saveFile( " + fileName + CLOSE_BRACKET;
        return false;
    }

    saveFile << s;
    saveFile.close();
    return true;
}

bool GizeMask::run(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e4: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[TO_BUFFER_SIZE];
    char to[TO_BUFFER_SIZE];
    if( in.getline(c, TO_BUFFER_SIZE) ) {
// Line-1
        if( strlen(c) > STANDARD_LINE_LENGTH ) {
            dividing(c, to, TO_BUFFER_SIZE);
            for(int i = 0; i < DRAW_ARRAY_SIZE; ++i) {
                draw[i] = a[i];
            }

        } else {
            const string LINE_LENGTH_ERROR("e5(Line-1): strlen(c) <= STANDARD_LINE_LENGTH");
            error = LINE_LENGTH_ERROR;
            return false;
        }

    } else {
        const string READ_ERROR("e6(Line-1): in.getline(c, TO_BUFFER_SIZE) ");
        error = READ_ERROR;
        return false;
    }

    int b, howMany, r;
    string *p = new string("");
    string extra, s;
    stringstream *os = new stringstream();
    Characterizable *lineType = new DateFollowed(*p);

    if(whoText) {
        textName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        textName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        scope = allQP(c);

        if( strlen(c) > STANDARD_LINE_LENGTH ) {
            delete p;
            p = new string(c);
            s = *p;

            delete lineType;
            lineType = new DateFollowed(s);
            r = lineType->dateLine();
            if(r > 0) {
                listing();

                id = lineType->name_id();
                res_no = lineType->anchor_res_no();
                clear(id, res_no, s, r);
                continue;
            }

            dividing(c, to, TO_BUFFER_SIZE);
        } else {
            continue;
        }

        b = 0;
        for(int i = 1; i <= CELL_COUNT; ++i) {
            if(i == 5) { // cell-5 is FREE.
                continue;
            }

            if(a[i] == draw[i]) {
                b |= shift[i];
            }

        }

        howMany = and_mask_equals_mask(b); // unbroken[]
        if(howMany == lines) {
            delete os;
            os = new stringstream();
            for(int i = 0; i < MASK_COUNT; ++i) {
                if(unbroken[i]) {
                    *os << trace[i] << COMMA_GAP;
                }

            }

            extra = os->str();
            s = assembling(extra);
            add(s);
        }

    }

    in.close();
    listing();
    openSaveClose(textName);

    delete lineType;
    delete os;
    delete p;
    return true;
}

GizeMask::GizeMask(int l, bool c, bool n, bool s, bool w): ALL_QP("ALL QP"), ALL_QP_SIZE(6), ANCHOR(">>"), CLOSE_BRACKET(" )"), COMMA_GAP(", "), EMPTY(""), FREE("**"), GAP(" "), GAP_LINES(" lines"), GAP_SLASH(" /"), GAP_SLASH_SLASH(" //"), ID_COLON(" ID:"), LINES(2), NEW_LINE("\n"), NEW_LINE_LENGTH(1), QP("QP"), SLASH("/"), SPACE("@"), TRACE_0("1-2-3"), TRACE_1("7-8-9"), TRACE_2("1-4-7"), TRACE_3("3-6-9"), TRACE_4("4-6"), TRACE_5("2-8"), TRACE_6("1-9"), TRACE_7("3-7") { // Shift_JIS
    lines = l;
    cell_5 = c;
    noSplitting = n;
    serialNumber = s;
    whoText = w;

    if(lines < 1 || 8 < lines) {
        lines = LINES;
    }

    buffer = new std::stringstream();
    combos = NULL;
    oneMore = false;
    std::string clearing = EMPTY;
    clear(clearing, clearing, clearing, 0); // combos = new std::stringstream();

    mask[0] = A123; trace[0] = TRACE_0;
    mask[1] = A789; trace[1] = TRACE_1;
    mask[2] = A147; trace[2] = TRACE_2;
    mask[3] = A369; trace[3] = TRACE_3;
    mask[4] = A46;  trace[4] = TRACE_4;
    mask[5] = A28;  trace[5] = TRACE_5;
    mask[6] = A19;  trace[6] = TRACE_6;
    mask[7] = A37;  trace[7] = TRACE_7;

    shift[0] = B00000000000000000000000000000000;
    shift[1] = B00000000000000000000000000000001;
    shift[2] = B00000000000000000000000000000010;
    shift[3] = B00000000000000000000000000000100;
    shift[4] = B00000000000000000000000000001000;
    shift[5] = B00000000000000000000000000010000;
    shift[6] = B00000000000000000000000000100000;
    shift[7] = B00000000000000000000000001000000;
    shift[8] = B00000000000000000000000010000000;
    shift[9] = B00000000000000000000000100000000;
}

int GizeMask::and_mask_equals_mask(int a) {
    for(int i = 0; i < MASK_COUNT; ++i) {
        unbroken[i] = false;
    }

    int b;
    int true_count = 0;
    for(int i = 0; i < MASK_COUNT; ++i) {
        b = a & mask[i];
        if(b == mask[i]) {
            true_count += 1;
            unbroken[i] = true;
        } else {
            unbroken[i] = false;
        }

    }

    return true_count;
}

std::string GizeMask::assembling(std::string & whichLine) {
    using namespace std;

    stringstream os;
    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        switch(i) {
        case 3:
            os << a[i] << GAP_SLASH;
            break;
        case 5:
            if(cell_5) {
                os << a[i] << GAP;
            }

            break;
        case 6:
            os << a[i] << GAP_SLASH;
            break;
        case 9:
            os << a[i] << GAP_SLASH_SLASH;
            break;
        default:
            if( a[i].length() > 0 ) {
                os << a[i] << GAP;
            }

            break;
        }

    }

    int q = whichLine.rfind(COMMA_GAP);
    if(q != string::npos) {
        string which_line = whichLine.substr(0, q);
        os << which_line << SLASH;
    }

    string s = os.str();
    return s;
}

void GizeMask::dividing(char *p, char *to, int to_array_size) {
    using namespace std;

    char *from;
    int n, u;
    string *to_string = new string("");

    a[0] = EMPTY;
    memset(to, 0, to_array_size);
    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        if(i == 5) {
            a[5] = FREE;
            continue;
        }

        n = strspn(p, "0123456789");
        from = p;
        p += n;

        u = strcspn(p, "0123456789");
        p += u;

        memcpy(to, from, n);
        to[n] = '\0';
        delete to_string;
        to_string = new string(to);
        a[i] = *to_string;
    }

    delete to_string;
}

void GizeMask::dividing(std::string & s, char *to, int to_array_size) {
    using namespace std;

    char *from;
    char *p;
    char c[TO_BUFFER_SIZE];
    int n, u;
    string *to_string = new string("");

    strcpy(c, s.c_str() );
    p = &c[0];
    a[0] = EMPTY;
    memset(to, 0, to_array_size);
    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        if(i == 5) {
            a[5] = FREE;
            continue;
        }

        n = strspn(p, "0123456789");
        from = p;
        p += n;

        u = strcspn(p, "0123456789");
        p += u;

        memcpy(to, from, n);
        to[n] = '\0';
        delete to_string;
        to_string = new string(to);
        a[i] = *to_string;
    }

    delete to_string;
}

void GizeMask::listing() {
    using namespace std;

    bool pile = false;
    int q;
    string chopped, head, tail;
    string combosList = combos->str();

    if(split == 0) {
        *buffer << NEW_LINE << ANCHOR << whose << NEW_LINE << combosList;
        return;
    }

    switch(count) {
    case 0:
        return;
    case 1:
        break;
    default:
        pile = true;
        break;
    }

    if(pile || noSplitting) {
        if(oneMore) {
            *buffer << NEW_LINE;
            oneMore = false;
        }

        if(scope) {
            q = combosList.find(ALL_QP);
            if(q != string::npos) {
                *buffer << NEW_LINE << ANCHOR << whose << NEW_LINE << combosList;
            } else {
                q = combosList.length() - NEW_LINE_LENGTH;
                chopped = combosList.substr(0, q);
                *buffer << NEW_LINE << ANCHOR << whose << NEW_LINE << chopped << SPACE << ALL_QP << NEW_LINE;
            }

        } else {
            *buffer << NEW_LINE << ANCHOR << whose << NEW_LINE << combosList;
        }

    } else {
// Splitting the date line
        q = combosList.length() - NEW_LINE_LENGTH;
        chopped = combosList.substr(0, q);
        oneMore = true;

        if(scope) {
            q = chopped.find(ALL_QP);
            if(q != string::npos) {
                head = chopped.substr(0, q);
                q += ALL_QP_SIZE;
                tail = chopped.substr(q);
                chopped = head + tail;
            }

            *buffer << res_no << chopped << GAP << QP << id;
        } else {
            *buffer << res_no << chopped << id;
        }

    }

}

void GizeMask::twoMatrices(std::string secondLine, std::string whoTextLine1) {
    using namespace std;

    char to[TO_BUFFER_SIZE];

    dividing(whoTextLine1, to, TO_BUFFER_SIZE);
    for(int i = 0; i < DRAW_ARRAY_SIZE; ++i) {
        draw[i] = a[i];
        gc->draw[i] = a[i];
    }

    dividing(secondLine, to, TO_BUFFER_SIZE);
    for(int i = 0; i < DRAW_ARRAY_SIZE; ++i) {
        gc->a[i] = a[i];
    }

// Comparing
    int b = 0;
    for(int i = 1; i <= CELL_COUNT; ++i) {
        if(i == 5) { // cell-5 is FREE.
            continue;
        }

        if(a[i] == draw[i]) {
            b |= shift[i];
        }

    }

// Counting
    int howMany = and_mask_equals_mask(b); // unbroken[]
    ul = new UnbrokenLine(howMany);

// Registering if any lines are unbroken.
    stringstream os;
    for(int i = 0; i < MASK_COUNT; ++i) {
        if(unbroken[i]) {
            os << trace[i] << COMMA_GAP;
            ul->registering(i);
        }

    }

    os << GAP;
    std::string extra = os.str();
    string s = assembling(extra);

    ul->remarks(s);
}

void GizeMask::unbrokenLines() {
    using namespace std;

    string s = ul->howManyLines();
    cout << s << endl;

    s = ul->secondLineExtra();
    cout << s << endl;
    delete ul;
}
