#include "TableCsv.h"

extern GIZE_CODE *gc;

bool TableCsv::openSaveClose(std::string & fileName) {
    using namespace std;

// Trimming
    string s = buffer->str();
    int size = s.length();
    string c;
    for(int i = 0; i < size; i += NEW_LINE_LENGTH) {
        c = s.substr(i, NEW_LINE_LENGTH);
        if(NEW_LINE == c) {
            continue;
        } else {
            s = s.substr(i);
            break;
        }

    }

// Writing
    ofstream saveFile( fileName.c_str() );
    if( saveFile.fail() ) {
        error = "e7: " + fileName + CLOSE_BRACKET;
        return false;
    }

    saveFile << s;
    saveFile.close();
    return true;
}

bool TableCsv::sorting(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e8: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[TO_BUFFER_SIZE];
    priority_queue<string> pq;

    string *p = new string(c);
    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        pq.push(*p);
    }

    stringstream ss;
    while(! pq.empty() ) {
		ss << pq.top() << NEW_LINE;
        pq.pop();
    }

    string s = ss.str();
    if(whoText) {
        textName = WHO_TEXT;
    } else {
        TimeName wt(WHO, CSV_FILE);
        textName = wt.temporaryName(serialNumber);
        gc->textName = textName;
    }

// Writing
    ofstream file( textName.c_str() );
    file << s;
    file.close();
    return true;
}

int TableCsv::and_mask_equals_mask(int a) {
    using namespace std;

    for(int i = 0; i < MASK_COUNT; ++i) {
        unbroken[i] = false;
    }

    int b;
    int true_count = 0;
    for(int i = 0; i < MASK_COUNT; ++i) {
        b = a & mask[i];
        if(b == mask[i]) {
            true_count += 1;
            unbroken[i] = true;
        } else {
            unbroken[i] = false;
        }

    }

    return true_count;
}

std::string TableCsv::assembling() {
    using namespace std;

    stringstream os;
    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        switch(i) {
        case 3:
            os << a[i] << GAP_SLASH;
            break;
        case 5:
            if(cell_5) {
                os << a[i] << GAP;
            }

            break;
        case 6:
            os << a[i] << GAP_SLASH;
            break;
        case 9:
            os << a[i] << GAP_SLASH_SLASH;
            break;
        default:
            if( a[i].length() > 0 ) {
                os << a[i] << GAP;
            }

            break;
        }

    }

    string s = os.str();
    return s;
}

void TableCsv::dividing(char *p, char *to, int to_array_size) {
    using namespace std;

    char *from;
    int n, u;
    string *to_string = new string("");

    a[0] = EMPTY;
    memset(to, 0, to_array_size);
    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        if(i == 5) {
            a[5] = FREE;
            continue;
        }

        n = strspn(p, "0123456789");
        from = p;
        p += n;

        u = strcspn(p, "0123456789");
        p += u;

        memcpy(to, from, n);
        to[n] = '\0';
        delete to_string;
        to_string = new string(to);
        a[i] = *to_string;
    }

    delete to_string;
}

void TableCsv::listing() {
    using namespace std;

    if(split == 0) {
        return;
    }

    switch(count) {
    case 0:
        return;
    default:
        break;
    }

// "Lines", "Bingo-5", "QP", "Date", "ID", "Res-number", "Name"

    int q;
    string head;
    string tail = combos->str();
    for(;;) {
        q = tail.find(SEPARATOR);
        if(q != string::npos) {
            head = tail.substr(0, q);
            q += SEPARATOR_SIZE;
            tail = tail.substr(q);

            if(scope) {
                *buffer << head << COMMA << ALL_QP << whose << NEW_LINE;
            } else {
                *buffer << head << COMMA << EMPTY << whose << NEW_LINE;
            }

            total += 1;
            q = strcspn( tail.c_str(), "0123456789");
            tail = tail.substr(q);
        } else {
            break;
        }

    }

}

TableCsv::TableCsv(bool c, bool s, bool w): ALL_QP("ALL QP"), ALL_QP_SIZE(6), CLOSE_BRACKET(" )"), COMMA(","), COMMA_GAP(", "), EMPTY(""), FREE("**"), GAP(" "), GAP_SLASH(" /"), GAP_SLASH_SLASH(" //"), ID_COLON(" ID:"), LINES(2), NEW_LINE("\n"), NEW_LINE_LENGTH(1), QP("QP"), SEPARATOR("`"), SEPARATOR_SIZE(1), SLASH("/"), SORT_IN_REVERSE_ORDER(true), TRACE_0("1-2-3"), TRACE_1("7-8-9"), TRACE_2("1-4-7"), TRACE_3("3-6-9"), TRACE_4("4-6"), TRACE_5("2-8"), TRACE_6("1-9"), TRACE_7("3-7"), TILDE("~") { // Shift_JIS
    buffer = new std::stringstream();
    cell_5 = c;
    combos = NULL;
    serialNumber = s;
    total = 0;
    whoText = w;

    std::string clearing = EMPTY;
    clear(clearing, 0); // combos = new std::stringstream();

    mask[0] = A123; trace[0] = TRACE_0;
    mask[1] = A789; trace[1] = TRACE_1;
    mask[2] = A147; trace[2] = TRACE_2;
    mask[3] = A369; trace[3] = TRACE_3;
    mask[4] = A46;  trace[4] = TRACE_4;
    mask[5] = A28;  trace[5] = TRACE_5;
    mask[6] = A19;  trace[6] = TRACE_6;
    mask[7] = A37;  trace[7] = TRACE_7;

    shift[0] = B00000000000000000000000000000000;
    shift[1] = B00000000000000000000000000000001;
    shift[2] = B00000000000000000000000000000010;
    shift[3] = B00000000000000000000000000000100;
    shift[4] = B00000000000000000000000000001000;
    shift[5] = B00000000000000000000000000010000;
    shift[6] = B00000000000000000000000000100000;
    shift[7] = B00000000000000000000000001000000;
    shift[8] = B00000000000000000000000010000000;
    shift[9] = B00000000000000000000000100000000;
}

bool TableCsv::run(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e9: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[TO_BUFFER_SIZE];
    char to[TO_BUFFER_SIZE];
    if( in.getline(c, TO_BUFFER_SIZE) ) {
// Line-1
        if( strlen(c) > STANDARD_LINE_LENGTH ) {
            dividing(c, to, TO_BUFFER_SIZE);
            for(int i = 0; i < DRAW_ARRAY_SIZE; ++i) {
                draw[i] = a[i];
                gc->draw[i] = a[i];
            }

        } else {
            const string LINE_LENGTH_ERROR("e10(Line-1): strlen(c) <= STANDARD_LINE_LENGTH");
            error = LINE_LENGTH_ERROR;
            return false;
        }

    } else {
        const string READ_ERROR("e11(Line-1): in.getline(c, TO_BUFFER_SIZE) ");
        error = READ_ERROR;
        return false;
    }

    int b, howMany, r;
    string *p = new string("");
    string s;
    Characterizable *lineType = new DateFollowed(*p);

    if(whoText) {
        const string who_text(WHO_TEXT);
        textName = who_text;
    } else {
        TimeName wt(WHO, CSV_FILE);
        textName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        scope = allQP(c);

        if( strlen(c) > STANDARD_LINE_LENGTH ) {
            delete p;
            p = new string(c);
            s = *p;

            delete lineType;
            lineType = new DateFollowed(s);
            r = lineType->dateLine();
            if(r > 0) {
                listing();
                s = lineType->common();
                clear(s, r);
                continue;
            }

            dividing(c, to, TO_BUFFER_SIZE);
        } else {
            continue;
        }

        b = 0;
        for(int i = 1; i <= 9; ++i) {
            if(i == 5) { // cell-5 is FREE.
                continue;
            }

            if(a[i] == draw[i]) {
                b |= shift[i];
            }

        }

        howMany = and_mask_equals_mask(b); // unbroken[]
        if(howMany > 0) {
            s = assembling();
            add(howMany, s);
        }

    }

    in.close();
    listing();
    openSaveClose(textName);
    sorting(textName);

    delete lineType;
    delete p;
    return true;
}
